local Headers = require 'luncheon.headers'
local normal_headers = require 'spec.normal_headers'

describe('Headers', function ()
    describe('append_chunk', function()
        it('All Standard headers', function()
            local h = Headers.new()
            for _, set in ipairs(normal_headers) do
                local chunk = set[1]
                local key = set[2]
                local expected = set[3]
                h:append_chunk(chunk)
                assert(h[key] == expected, string.format('%s found %s expected %s', key, h[key], expected))
            end
        end)
        it('from_chunk', function()
            local h = Headers.from_chunk('Accept: application/json')
            assert(h.accept == 'application/json', string.format('expected application/json, found %s', h.accept))
        end)
        it('append', function()
            local h = Headers.new()
            h:append('accept', 'application/json1')
            assert(h.accept == 'application/json1', string.format('expected application/json, found %s', h.accept))
            h:append('accept', 'application/json2')
            assert(h.accept[1] == 'application/json1', string.format('expected application/json1, found %s', h.accept))
            assert(h.accept[2] == 'application/json2', string.format('expected application/json2, found %s', h.accept))
        end)
        it('append_chunk', function()
            local h = Headers.new()
            local _, err1 = h:append_chunk('Accept: application/json')
            assert(err1 == nil, string.format('error frrom first append "%s"', err1))
            local _, err2 = h:append_chunk(' application/text')
            assert(err2 == nil, string.format('error frrom second append "%s"', err2))
            assert(h.accept == 'application/json  application/text', string.format('bad header "%s"', h.accept))
        end)
        it('bad append_chunk', function ()
            local h = Headers.new()
            local _, err1 = h:append_chunk(' bad')
            assert(err1 == 'Header continuation with no key', err1)
        end)
        it('append', function()
            local h = Headers.new()
            h:append('accept', 'application/json1')
            assert(h.accept == 'application/json1', string.format('expected application/json, found %s', h.accept))
            h:append('accept', 'application/json2')
            assert(h.accept[1] == 'application/json1', string.format('expected application/json1, found %s', h.accept))
            assert(h.accept[2] == 'application/json2', string.format('expected application/json2, found %s', h.accept))
            h:append('accept', 'application/json3')
            assert(h.accept[1] == 'application/json1', string.format('expected application/json1, found %s', h.accept))
            assert(h.accept[2] == 'application/json2', string.format('expected application/json2, found %s', h.accept))
            assert(h.accept[3] == 'application/json3', string.format('expected application/json3, found %s', h.accept))
        end)
        it('get_one', function()
            local h = Headers.new()
            h:append('accept', 'application/json1')
            assert(h:get_one('accept') == 'application/json1', string.format('expected application/json1, found %s', h:get_one('accept')))
            h:append('accept', 'application/json2')
            assert(h:get_one('accept') == 'application/json2', string.format('expected application/json1, found %s', h:get_one('accept')))
        end)
        it('get_all', function()
            local h = Headers.new()
            h:append('accept', 'application/json1')
            local table1 = h:get_all('accept')
            assert(table1[1] == 'application/json1', string.format('expected application/json1, found %s', table[1]))
            h:append('accept', 'application/json2')     
            local table2 = h:get_all('accept')
            assert(table2[1] == 'application/json1', string.format('expected application/json1, found %s', table[1]))
            assert(table2[2] == 'application/json2', string.format('expected application/json2, found %s', table[2]))
        end)
        it('Can handle multi line headers', function()
            local h = Headers.new()
            h:append_chunk('x-Multi-Line-Header: things and stuff')
            h:append_chunk(' places and people')
            assert(h.x_multi_line_header, 'thinigs and stuff\nplaces and people')
        end)
    end)
    describe('serialize_header', function ()
        it('can handle normal header', function()
            for _, set in ipairs(normal_headers) do
                local header = Headers.serialize_header(set[2], set[3])
                assert(header == set[1], string.format('expected %s found %s', set[1], header))
            end
        end)
        it('multiline header', function()
            local header1 = Headers.serialize_header('accept', {'application/json1'})
            assert(header1 == 'Accept: application/json1', string.format('expected application/json1, found %s', header1))
            local header2 = Headers.serialize_header('accept', {'application/json1', 'application/json2'})
            assert(header2 == 'Accept: application/json2', string.format('expected application/json2, found %s', header2))
        end)
    end)
end)